/*
 * Project: MoleCuilder
 * Description: creates and alters molecular systems
 * Copyright (C)  2010 University of Bonn. All rights reserved.
 * Please see the LICENSE file or "Copyright notice" in builder.cpp for details.
 */

/*
 * SetRandomNumbersEngine.cpp
 *
 *  Created on: Jan 01, 2011
 *      Author: heber
 */

// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "CodePatterns/MemDebug.hpp"

#include "CodePatterns/Log.hpp"
#include "molecule.hpp"
#include "CodePatterns/Verbose.hpp"
#include "World.hpp"

#include "RandomNumbers/RandomNumberGeneratorFactory.hpp"

#include <iostream>
#include <fstream>
#include <string>

using namespace std;

#include "SetRandomNumbersEngineAction.hpp"

// and construct the stuff
#include "SetRandomNumbersEngineAction.def"
#include "Action_impl_pre.hpp"
/** =========== define the function ====================== */
Action::state_ptr CommandSetRandomNumbersEngineAction::performCall() {
  // obtain information
  getParametersfromValueStorage();

  CommandSetRandomNumbersEngineState *state =
      new CommandSetRandomNumbersEngineState(
          RandomNumberGeneratorFactory::getInstance().getEngineName(),
          params);
  RandomNumberGeneratorFactory::getInstance().setEngine(params.engine_type);

  DoLog(0) && (Log() << Verbose(0) << "Engine of random number generator is now: "
      << RandomNumberGeneratorFactory::getInstance().getEngineName() << std::endl);

  return Action::state_ptr(state);
}

Action::state_ptr CommandSetRandomNumbersEngineAction::performUndo(Action::state_ptr _state) {
  CommandSetRandomNumbersEngineState *state = assert_cast<CommandSetRandomNumbersEngineState*>(_state.get());

  CommandSetRandomNumbersEngineState *newstate =
      new CommandSetRandomNumbersEngineState(
          RandomNumberGeneratorFactory::getInstance().getEngineName(),
          state->params);
  RandomNumberGeneratorFactory::getInstance().setEngine(state->old_engine_type);

  DoLog(0) && (Log() << Verbose(0) << "Engine of random number generator is undone to: "
      << RandomNumberGeneratorFactory::getInstance().getEngineName() << std::endl);

  return Action::state_ptr(newstate);
}

Action::state_ptr CommandSetRandomNumbersEngineAction::performRedo(Action::state_ptr _state){
  return performUndo(_state);
}

bool CommandSetRandomNumbersEngineAction::canUndo() {
  return true;
}

bool CommandSetRandomNumbersEngineAction::shouldUndo() {
  return true;
}
/** =========== end of function ====================== */
